"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutlineTreeService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_components_1 = require("@opensumi/ide-components");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const document_symbol_1 = require("@opensumi/ide-editor/lib/browser/breadcrumb/document-symbol");
const outline_node_define_1 = require("../outline-node.define");
const outline_contextkey_service_1 = require("./outline-contextkey.service");
let OutlineTreeService = class OutlineTreeService extends ide_components_1.Tree {
    constructor() {
        super();
        // 处理字符串排序，IdleValue 在空闲或需要时执行 [idle-or-urgent stratage implementation](https://philipwalton.com/articles/idle-until-urgent)
        this.collator = new ide_core_browser_1.IdleValue(() => new Intl.Collator(undefined, { numeric: true }));
        this.cacheOutlineNodes = new Map();
        this._sortType = 0 /* OutlineSortOrder.ByPosition */;
        this._followCursor = false;
        this.onDidChangeEmitter = new ide_core_browser_1.Emitter();
        this.sortComparator = (a, b) => {
            if (this.sortType === 2 /* OutlineSortOrder.ByKind */) {
                return (a.raw.kind - b.raw.kind ||
                    this.collator.getValue().compare(a.displayName, b.displayName));
            }
            else if (this.sortType === 1 /* OutlineSortOrder.ByName */) {
                return (this.collator.getValue().compare(a.displayName, b.displayName) ||
                    (0, ide_core_browser_1.compareRangesUsingStarts)(a.raw.range, b.raw.range));
            }
            else {
                return ((0, ide_core_browser_1.compareRangesUsingStarts)(a.raw.range, b.raw.range) ||
                    this.collator.getValue().compare(a.displayName, b.displayName));
            }
        };
        this._whenReady = this.init();
    }
    async init() {
        this._outlineStorage = await this.storageProvider(ide_core_browser_1.STORAGE_NAMESPACE.OUTLINE);
        this._sortType = this._outlineStorage.get('sortType', 0 /* OutlineSortOrder.ByPosition */);
        this.outlineContextKeyService.outlineSortTypeContext.set(this._sortType);
        this._followCursor = this._outlineStorage.get('followCursor', false);
        this.outlineContextKeyService.outlineFollowCursorContext.set(this._followCursor);
    }
    get whenReady() {
        return this._whenReady;
    }
    /**
     * trigger when outline tree changed(include sortType, followCursor changed)
     */
    get onDidChange() {
        return this.onDidChangeEmitter.event;
    }
    get currentUri() {
        return this._currentUri;
    }
    set currentUri(value) {
        this._currentUri = value;
    }
    get followCursor() {
        return this._followCursor;
    }
    set followCursor(value) {
        if (this._followCursor === value) {
            return;
        }
        this._followCursor = value;
        this.outlineContextKeyService.outlineFollowCursorContext.set(value);
        this._outlineStorage.set('followCursor', value);
        this.onDidChangeEmitter.fire();
    }
    get sortType() {
        return this._sortType;
    }
    set sortType(type) {
        if (this._sortType === type) {
            return;
        }
        this._sortType = type;
        this.outlineContextKeyService.outlineSortTypeContext.set(type);
        this._outlineStorage.set('sortType', type);
        this.onDidChangeEmitter.fire();
    }
    async resolveChildren(parent) {
        var _a;
        let children = [];
        if (!parent) {
            children = [new outline_node_define_1.OutlineRoot(this, this.currentUri)];
        }
        else if (parent) {
            if (!outline_node_define_1.OutlineCompositeTreeNode.is(parent)) {
                if (!(parent.currentUri || this.currentUri)) {
                    return [];
                }
                const symbols = this.documentSymbolStore.getDocumentSymbol((parent.currentUri || this.currentUri));
                children =
                    (symbols === null || symbols === void 0 ? void 0 : symbols.map((symbol) => {
                        var _a;
                        if ((_a = symbol.children) === null || _a === void 0 ? void 0 : _a.length) {
                            return new outline_node_define_1.OutlineCompositeTreeNode(this, parent, symbol, (0, ide_core_browser_1.getSymbolIcon)(symbol.kind) + ' outline-icon');
                        }
                        return new outline_node_define_1.OutlineTreeNode(this, parent, symbol, (0, ide_core_browser_1.getSymbolIcon)(symbol.kind) + ' outline-icon');
                    })) || [];
            }
            else if (parent.raw) {
                children =
                    ((_a = parent.raw.children) === null || _a === void 0 ? void 0 : _a.map((symbol) => {
                        var _a;
                        if ((_a = symbol.children) === null || _a === void 0 ? void 0 : _a.length) {
                            return new outline_node_define_1.OutlineCompositeTreeNode(this, parent, symbol, (0, ide_core_browser_1.getSymbolIcon)(symbol.kind) + ' outline-icon');
                        }
                        return new outline_node_define_1.OutlineTreeNode(this, parent, symbol, (0, ide_core_browser_1.getSymbolIcon)(symbol.kind) + ' outline-icon');
                    }).filter((node) => !!node.name)) || [];
            }
        }
        this.cacheNodes(children);
        return children;
    }
    cacheNodes(nodes) {
        for (const node of nodes) {
            if (node.raw) {
                // OutlineTreeNode or OutlineCompositeTreeNode
                this.cacheOutlineNodes.set(node.raw.id, node);
            }
        }
    }
    getTreeNodeBySymbol(symbol) {
        if (symbol) {
            const cache = this.cacheOutlineNodes.get(symbol.id);
            return cache;
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(document_symbol_1.DocumentSymbolStore),
    tslib_1.__metadata("design:type", document_symbol_1.DocumentSymbolStore)
], OutlineTreeService.prototype, "documentSymbolStore", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(outline_contextkey_service_1.OutlineContextKeyService),
    tslib_1.__metadata("design:type", outline_contextkey_service_1.OutlineContextKeyService)
], OutlineTreeService.prototype, "outlineContextKeyService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.StorageProvider),
    tslib_1.__metadata("design:type", Function)
], OutlineTreeService.prototype, "storageProvider", void 0);
OutlineTreeService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], OutlineTreeService);
exports.OutlineTreeService = OutlineTreeService;
//# sourceMappingURL=outline-tree.service.js.map